'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
// target
var defineName = 'Promise';
var methods = ['resolve', 'reject', 'all', 'race'];

/**
* @module babel-plugin-transform-bluebird
* @returns {babelPlugin} unknown
*/

exports.default = function () {
  return {
    visitor: {
      Program: {
        enter: function enter(path, file) {
          file.__UNUSE_BLUEBIRD__ = false;
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = path.node.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var node = _step.value;

              // found "import Promise"
              var specifiers = node.specifiers || [];
              var localNames = specifiers.map(function (specifier) {
                return specifier.local.name;
              });
              if (localNames.indexOf(defineName) > -1) {
                file.__UNUSE_BLUEBIRD__ = true;
                return;
              }

              // found "var Promise"
              var declarations = node.declarations || [];
              var declarationIds = declarations.map(function (declaration) {
                return declaration.id.name;
              });
              if (declarationIds.indexOf(defineName) > -1) {
                file.__UNUSE_BLUEBIRD__ = true;
                return;
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }
        }
      },

      // found "new Promise"
      NewExpression: function NewExpression(path, file) {
        if (file.__UNUSE_BLUEBIRD__) {
          return;
        }

        if (path.get('callee').node.name === defineName) {
          path.node.callee = file.addImport('bluebird', 'default');
        }
      },


      // found "Promise.methods()"
      CallExpression: function CallExpression(path, file) {
        if (file.__UNUSE_BLUEBIRD__) {
          return;
        }

        methods.forEach(function (method) {
          if (path.get('callee').matchesPattern(defineName + '.' + method)) {
            path.node.callee = file.addImport('bluebird', method);
          }
        });
      }
    }
  };
};

module.exports = exports['default'];
//# sourceMappingURL=index.js.map